GenEngine engine;

int visNumRows;
int visNumCols;
int visIndSize;

// constants for visualization
int visDimensions = 800;
int visIndPad = 10;
int visOffset;

// Setup
void setup () {
  // create window
  size (visDimensions, visDimensions);
  background(255);
  
  // initialize genetic sim engine
  engine = new GenEngine();
  engine.generateInitPopulation();
  
  // visualization config
  visNumRows = (int) Math.ceil(Math.sqrt(GenEngine.popSize));
  visNumCols = (int) Math.ceil(GenEngine.popSize / (float) visNumRows);
  visIndSize = Math.round((visDimensions - visDimensions / (float) 20) / visNumRows) - 10;
  visOffset =  (int) Math.round( visIndSize / (float) 1.5);
  
  noLoop();
}

// Draw visualization
void draw() {
  int curIndex;
  Individual curInd;
  
  int x, y;
  
  // Loop through every individual and display (in rows/columns)
  for (int i = 0; i < visNumRows; i++) {
    for (int j = 0; j < visNumCols; j++) {
      curIndex = i * visNumCols + j;  // calculate current index by current row/col
      
      if (curIndex >= GenEngine.popSize)  // done if population size exceeded
        break;
      
      // Retrieve individual + draw
      curInd = (Individual) engine.population.get(curIndex);
      
      x = j * (visIndSize + visIndPad) + visOffset;
      y = i * (visIndSize + visIndPad) + visOffset;
      
      fill(curInd.tag * 255);
      ellipse(x, y, visIndSize, visIndSize);
      fill((curInd.tolerance * 10) * 255);
      ellipse(x, y, visIndSize / 2, visIndSize / 2);
    }
  }
}

void keyPressed(){
  if ((key == '1') || (key == 'o') || (key == 'O')){
    skipGenerations(1);
    redraw();
  }
  if (key == '5'){
    skipGenerations(5);
    redraw();
  }
  if ((key == 't') || (key == 'T')){
    skipGenerations(10);
    redraw();
  }
  if ((key == 'f') || (key == 'F')){
    skipGenerations(50);
    redraw();
  }
  if ((key == 'h') || (key == 'H')){
    skipGenerations(100);
    redraw();
  }
}

void skipGenerations(int gens){
  for (int i = 0; i < gens; i++){
    engine.runPopulation();
    println("score: " + engine.averageScore() + " tol " + engine.averageTolerance() + "d % " + engine.donationRate() * 100);
    engine.generatePopulation();
  }
}
